package mytv.model;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class Schedule {
	private LocalDateTime startTime;
	private List<ScheduledProgramme> programmes;

	public Schedule(LocalDateTime startTime) {
		if (startTime == null)
			throw new IllegalArgumentException("startTime is null");
		
		this.startTime = startTime;
		this.programmes = new ArrayList<>();
	}
	
	public LocalDateTime getStartTime() {
		return startTime;
	}

	public LocalDateTime getEndTime() {
		return programmes.isEmpty()
				? getStartTime()
				: programmes.get(programmes.size() - 1).getEndTime();
	}
	
	public List<ScheduledProgramme> getScheduledProgrammes() {
		return Collections.unmodifiableList(this.programmes);
	}
	
	public void add(Programme programme) {
		ScheduledProgramme toAdd;
		if (programmes.isEmpty()) {
			toAdd = new ScheduledProgramme(programme, getStartTime());
		} else {
			LocalDateTime programmeStart = programmes.get(programmes.size() - 1).getEndTime();
			toAdd = new ScheduledProgramme(programme, programmeStart);
		}
		programmes.add(toAdd);
	}
	
	public void insertBefore(ScheduledProgramme reference, Programme programme) {
		if (reference == null)
			throw new IllegalArgumentException("reference is null");
		
		int insertIndex = programmes.indexOf(reference);
		if (insertIndex < 0)
			throw new IllegalArgumentException("reference not in schedule");
		
		ScheduledProgramme toAdd;
		if (insertIndex == 0) {
			toAdd = new ScheduledProgramme(programme, getStartTime());
		} else {
			LocalDateTime programmeStart = programmes.get(insertIndex - 1).getEndTime();
			toAdd = new ScheduledProgramme(programme, programmeStart);
		}
		
		programmes.add(insertIndex, toAdd);
		rescheduleAfter(insertIndex);
	}

	public void insertAfter(ScheduledProgramme reference, Programme programme) {
		if (reference == null)
			throw new IllegalArgumentException("reference is null");
		
		int insertIndex = programmes.indexOf(reference);
		if (insertIndex < 0)
			throw new IllegalArgumentException("reference not in schedule");
		
		if (insertIndex == programmes.size() - 1) {
			add(programme);
		} else {
			LocalDateTime programmeStart = programmes.get(insertIndex).getEndTime();
			ScheduledProgramme toAdd = new ScheduledProgramme(programme, programmeStart);
			insertIndex++;	// insert after...
			programmes.add(insertIndex, toAdd);
			rescheduleAfter(insertIndex);
		}
	}
	
	public ScheduledProgramme remove(int index) {
		ScheduledProgramme removed = programmes.remove(index);
		if (index == 0 && programmes.size() > 0) {
			programmes.get(0).setStartTime(getStartTime());
			rescheduleAfter(0);
		} else {
			rescheduleAfter(index - 1);
		}
		return removed;
	}
	
	private void rescheduleAfter(int index) {
		for (int i = index + 1; i < programmes.size(); i++) {
			ScheduledProgramme toReschedule = programmes.get(i);
			ScheduledProgramme reference = programmes.get(i - 1);
			toReschedule.setStartTime(reference.getEndTime());
		}
	}
}
