package mytv.ui;

import java.time.Duration;
import java.time.LocalDateTime;
import java.util.List;
import java.util.SortedSet;

import mytv.model.Programme;
import mytv.model.Schedule;
import mytv.model.ScheduledProgramme;

public class MyController implements Controller {

	private SortedSet<Programme> allPrograms;
	private Schedule schedule;
	
	public MyController(SortedSet<Programme> allPrograms) {
		if (allPrograms == null)
			throw new IllegalArgumentException("stations");
		this.allPrograms=allPrograms;
		
//		createScheduleStartingFrom(LocalDateTime.now());
	}
	
	@Override
	public void createScheduleStartingFrom(LocalDateTime startTime) {
		schedule = new Schedule(startTime);
	}
	
	@Override
	public boolean isScheduleCreated() {
		return schedule != null;
	}

	@Override
	public SortedSet<Programme> getAllProgrammes() {
		return allPrograms;
	}

	@Override
	public void addProgramme(Programme programme) {
		checkScheduleCreated();
		
		schedule.add(programme);
	}

	@Override
	public List<ScheduledProgramme> getCurrentProgrammeSchedule() {
		checkScheduleCreated();
		return schedule.getScheduledProgrammes();
	}

	private void checkScheduleCreated() {
		if (!isScheduleCreated()) {
			throw new IllegalStateException("Create a schedule first");
		}
	}

	@Override
	public Duration getTotalDuration() {
		checkScheduleCreated();
		
		return Duration.between(schedule.getStartTime(), schedule.getEndTime());
	}
	
	@Override
	public ScheduledProgramme removeProgramme(int position) {
		checkScheduleCreated();
		
		return schedule.remove(position);
	}
}
