package mytv.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import mytv.model.Programme;
import mytv.model.ScheduledProgramme;
import mytv.util.DurationHelpers;

public class SchedulePanel extends JPanel implements ListSelectionListener,
		ActionListener {

	private static final long serialVersionUID = 1L;

	private static Dimension labelCommonDimension = new Dimension(170, 20);

	private Controller controller;

	private JComboBox<Programme> programmesCombo;
	private JTextArea scheduleOutput;
	private ScheduleList currentSchedule;

	public SchedulePanel(Controller controller) {
		this.controller = controller;
		initGUI();
		bindData();
	}

	private void initGUI() {
		setLayout(new BorderLayout(15, 15));

		JPanel pageStartPanel = new JPanel(new BorderLayout());
		{
			JLabel label1 = new JLabel("Programmi disponibili:");
			label1.setPreferredSize(labelCommonDimension);
			pageStartPanel.add(label1, BorderLayout.LINE_START);

			programmesCombo = new JComboBox<>();
			pageStartPanel.add(programmesCombo, BorderLayout.CENTER);
		}
		add(pageStartPanel, BorderLayout.PAGE_START);

		JPanel centerPanel = new JPanel(new BorderLayout());
		{
			JLabel label2 = new JLabel("Palinsesto corrente:");
			label2.setPreferredSize(labelCommonDimension);
			centerPanel.add(label2, BorderLayout.LINE_START);

			currentSchedule = new ScheduleList();
			currentSchedule.addListSelectionListener(this);

			centerPanel.add(new JScrollPane(currentSchedule),
					BorderLayout.CENTER);
		}
		add(centerPanel, BorderLayout.CENTER);

		JPanel pageEndPanel = new JPanel(new FlowLayout());
		{
			scheduleOutput = new JTextArea(1, 22);
			scheduleOutput.setFont(new Font("Verdana", Font.BOLD, 12));
			scheduleOutput.setForeground(Color.white);
			scheduleOutput.setBackground(Color.blue);
			scheduleOutput.setEditable(false);
			pageEndPanel.add(scheduleOutput);
		}
		add(pageEndPanel, BorderLayout.PAGE_END);
	}

	private void bindData() {
		for (Programme p : controller.getAllProgrammes()) {
			programmesCombo.addItem(p);
		}
		programmesCombo.addActionListener(this);
		update();
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		Programme p = (Programme) programmesCombo.getSelectedItem();
		if (p == null)
			return;
		controller.addProgramme(p);
		update();
	}

	@Override
	public void valueChanged(ListSelectionEvent ev) {
		ScheduledProgramme selected = currentSchedule.getSelectedValue();
		if (selected == null)
			return;

		int selectedIndex = currentSchedule.getSelectedIndex();
		int confirm = JOptionPane.showConfirmDialog(this, "Rimuovere "
				+ selected.toString() + " alla posizione "
				+ (selectedIndex + 1) + " ?", "Attenzione", JOptionPane.YES_NO_OPTION);
		if (confirm == JOptionPane.YES_OPTION) {
			controller.removeProgramme(selectedIndex);
			update();
		}
	}

	@Override
	public void setEnabled(boolean enabled) {
		super.setEnabled(enabled);

		programmesCombo.setEnabled(enabled);
		currentSchedule.setEnabled(enabled);
	}

	private void update() {
		if (!controller.isScheduleCreated())
			return;
		
		currentSchedule.load(controller.getCurrentProgrammeSchedule());
		
		String duration = DurationHelpers.formatDuration(controller
				.getTotalDuration());
		scheduleOutput.setText("Durata totale: " + duration);
	}

}
